/*
	Script for Oracle to upgrade clone items.
	After installing hotfix all existing clone items must have additional shared field __Source item. 
	Value for this field determined by values in __Source fields for all item's versions. Upgrade happens for consistent clones only. 
	
	Clone considered as inconsistent when item has different IDs among different versions in __Source field.

	For inconsistent items upgrade process must be done manualy.

	All ids of inconsistent items will be output during script execution.
*/

DECLARE
  tCount PLS_INTEGER;
BEGIN
  SELECT COUNT(*) INTO tCount from user_tables where table_name='TempItemsWithSources';

  IF( tCount > 0 ) THEN 
    EXECUTE IMMEDIATE 'DROP TABLE "TempItemsWithSources"';
  END IF;
END;
/ 

CREATE TABLE "TempItemsWithSources" AS (SELECT "ItemId", "Value", "Language", "Version" FROM "VersionedFields" WHERE "FieldId" = '7D69861BCA60804D83FB7555A2E6CE1C' AND  length("Value") > 0);
COMMIT;
DECLARE
  sourceFieldId CONSTANT NVARCHAR2(50) := '7D69861BCA60804D83FB7555A2E6CE1C';
  sourceItemFieldId CONSTANT NVARCHAR2(50) := 'D397B519DD2EE24AAEFE4A94C7F10E31';
  rCount number;
  itemId NVARCHAR2(100);
  val NVARCHAR2(100);
  lang NVARCHAR2(100);
  version number;
  sourcesCount number;
  sourcesCountWithSameItem number;
  sourceItemValue NVARCHAR2(100);
  sourcesCountValidLang number;
BEGIN
  DBMS_OUTPUT.ENABLE(1000000);
  
  SELECT COUNT(*) INTO rCount FROM "TempItemsWithSources";
 
  WHILE rCount > 0 LOOP

    SELECT "ItemId", "Value", "Language", "Version" INTO itemId, val, lang, version FROM "TempItemsWithSources" WHERE ROWNUM = 1;
	
    dbms_output.put_line('Processing ' || itemId || ' item. Language: ' || lang || '. Version: ' || TO_CHAR(version) || '. Value: ' || val || '.');

    IF INSTR(val, '?') -1 <= 0 THEN
      dbms_output.put_line('Value of _Source field is invalid. Getting next item.');
      DELETE FROM "TempItemsWithSources" WHERE "ItemId" = itemId;
      SELECT COUNT(*) INTO rCount FROM "TempItemsWithSources";

      CONTINUE;
    END IF;

    sourceItemValue := SUBSTR(val, 1, INSTR(val, '?') -1);

    SELECT COUNT(*) INTO sourcesCount FROM "TempItemsWithSources" WHERE "ItemId" = itemId;
    SELECT COUNT(*) INTO sourcesCountWithSameItem FROM "TempItemsWithSources" WHERE "ItemId" = itemId AND "Value" LIKE sourceItemValue || '%';
  
    IF sourcesCount != sourcesCountWithSameItem THEN
      dbms_output.put_line('Item is not consistent. __Source field has different item id in different versions.');
      DELETE FROM "TempItemsWithSources" WHERE "ItemId" = itemId;
      SELECT COUNT(*) INTO rCount FROM "TempItemsWithSources";

      CONTINUE;
    END IF;

    dbms_output.put_line('Item is valid clone. Adding __Source Item shared field.');

    SELECT COUNT(*) INTO sourcesCount FROM "SharedFields" WHERE "ItemId" = itemId and "FieldId" = sourceItemFieldId;
	
    IF sourcesCount = 0 THEN
      INSERT INTO "SharedFields" ("Id", "ItemId", "FieldId", "Value", "Created", "Updated")
        VALUES (SYS_GUID(), itemId, sourceItemFieldId, sourceItemValue, SYSDATE, SYSDATE);
  	COMMIT;
      dbms_output.put_line('Added.');
    ELSE 
      dbms_output.put_line('Shared field __Source item with id ' || sourceItemFieldId || 'already exists for item ' || itemId || '. New row not added.');
    END IF;
  
    DELETE FROM "TempItemsWithSources" WHERE "ItemId" = itemId;
    SELECT COUNT(*) INTO rCount FROM "TempItemsWithSources";
  END LOOP;
END;
/
DROP TABLE "TempItemsWithSources";
COMMIT;